@@ -1,6 +1,6 @@
 package DBI::Gofer::Execute;
 
-#   $Id: Execute.pm 9537 2007-05-09 20:18:11Z timbo $
+#   $Id: Execute.pm 9838 2007-08-15 16:23:37Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -16,7 +16,7 @@ use DBI::Gofer::Response;
 
 use base qw(DBI::Util::_accessor);
 
-our $VERSION = sprintf("0.%06d", q$Revision: 9537 $ =~ /(\d+)/o);
+our $VERSION = sprintf("0.%06d", q$Revision: 9838 $ =~ /(\d+)/o);
 
 our @all_dbh_methods = sort map { keys %$_ } $DBI::DBI_methods{db}, $DBI::DBI_methods{common};
 our %all_dbh_methods = map { $_ => DBD::_::db->can($_) } @all_dbh_methods;
@@ -41,10 +41,12 @@ my %configuration_attributes = (
     forced_connect_attributes  => {},
     track_recent => 1,
     check_request_sub => sub {},
+    check_response_sub => sub {},
     forced_single_resultset => 1,
     max_cached_dbh_per_drh => 1,
     max_cached_sth_per_dbh => 1,
     forced_response_attributes => {},
+    forced_gofer_random => 1,
     stats => {},
 );
 
@@ -164,6 +166,13 @@ sub _connect {
     $dsn = $self->forced_connect_dsn || $dsn || $self->default_connect_dsn
         or die "No forced_connect_dsn, requested dsn, or default_connect_dsn for request";
 
+    # ensure this connect_cached doesn't have the same args as the client
+    # because that causes subtle issues if in the same process (ie transport=null)
+    # include pid to avoid problems with forking (ie null transport in mod_perl)
+    # include gofer-random to avoid random behaviour leaking to other handles
+    my $extra_cache_key = join "|",
+        __PACKAGE__, "$$", $self->{forced_gofer_random} || $ENV{DBI_GOFER_RANDOM} || '';
+
     # XXX implement our own private connect_cached method? (with rate-limited ping)
     my $dbh = DBI->$connect_method($dsn, undef, undef, {
 
@@ -192,18 +201,17 @@ sub _connect {
         # if errors happened before the main part of the request was executed
         Executed => 0,
 
-        # ensure this connect_cached doesn't have the same args as the client
-        # because that causes subtle issues if in the same process (ie transport=null)
-	# include pid to avoid problems with forking (ie null transport in mod_perl)
-        dbi_go_execute_unique => __PACKAGE__."$$",
+        # ensure connect_cached is sufficiently distinct
+        dbi_go_execute_unique => $extra_cache_key,
     });
 
     $dbh->{ShowErrorStatement} = 1 if $local_log;
 
-    # note that this affects previously cached handles because $ENV{DBI_GOFER_RANDOM}
-    # isn't included in the cache key. Could add a go_rand_fail=>... attribute.
-    $self->_install_rand_callbacks($dbh, $ENV{DBI_GOFER_RANDOM})
-        if $ENV{DBI_GOFER_RANDOM};
+    # XXX should probably just be a Callbacks => arg to connect_cached
+    # with a cache of pre-built callback hoks (memoized, without $self) 
+    if (my $random = $self->{forced_gofer_random} || $ENV{DBI_GOFER_RANDOM}) {
+        $self->_install_rand_callbacks($dbh, $random);
+    }
 
     my $CK = $dbh->{CachedKids};
     if ($CK && keys %$CK > $self->{max_cached_sth_per_dbh}) {
@@ -266,7 +274,7 @@ sub execute_request {
     my $response = eval {
 
         if (my $check_request_sub = $self->check_request_sub) {
-            $request = $check_request_sub->($request)
+            $request = $check_request_sub->($request, $self)
                 or die "check_request_sub failed";
         }
 
@@ -279,6 +287,12 @@ sub execute_request {
             : $self->execute_dbh_request($request);
     };
     $response ||= $self->new_response_with_err(undef, $@, $current_dbh);
+
+    if (my $check_response_sub = $self->check_response_sub) {
+        eval { $check_response_sub->($response, $self, $request) };
+        warn "check_response_sub failed: $@" if $@;
+    }
+
     undef $current_dbh;
 
     $response->warnings(\@warnings) if @warnings;
@@ -294,7 +308,7 @@ sub execute_dbh_request {
     my $dbh;
     my $rv_ref = eval {
         $dbh = $self->_connect($request);
-        my $args = $request->dbh_method_call; # [ 'method_name', @args ]
+        my $args = $request->dbh_method_call; # [ wantarray, 'method_name', @args ]
         my $wantarray = shift @$args;
         my $meth      = shift @$args;
         $stats->{method_calls_dbh}->{$meth}++;
@@ -556,11 +570,11 @@ sub _install_rand_callbacks {
     my ($fail_percent, $delay_percent, $delay_duration);
     my @specs = split /,/, $dbi_gofer_random;
     for my $spec (@specs) {
-        if ($spec =~ m/^fail=([.\d]+)%?$/) {
+        if ($spec =~ m/^fail=(-?[.\d]+)%?$/) {
             $fail_percent = $1;
             next;
         }
-        if ($spec =~ m/^delay([.\d]+)=([.\d]+)%?$/) {
+        if ($spec =~ m/^delay([.\d]+)=(-?[.\d]+)%?$/) {
             $delay_duration = $1;
             $delay_percent  = $2;
             next;
@@ -585,22 +599,33 @@ sub _install_rand_callbacks {
     $dbh->{private_gofer_rand_fail_callbacks} = $callbacks;
 }
 
+my %_mk_rand_callback_seqn;
 
 sub _mk_rand_callback {
     my ($self, $method, $fail_percent, $delay_percent, $delay_duration) = @_;
+    my ($fail_modrate, $delay_modrate);
+    $fail_percent  ||= 0;  $fail_modrate  = int(1/(-$fail_percent )*100) if $fail_percent;
+    $delay_percent ||= 0;  $delay_modrate = int(1/(-$delay_percent)*100) if $delay_percent;
     # note that $method may be "*"
     return sub {
         my ($h) = @_;
-        if ($delay_percent && rand(100) < $delay_percent) {
-            my $msg = "DBI_GOFER_RANDOM delaying execution of $method by $delay_duration seconds\n";
+        my $seqn = ++$_mk_rand_callback_seqn{$method};
+        my $delay = ($delay_percent > 0) ? rand(100) < $delay_percent :
+                    ($delay_percent < 0) ? !($seqn % $delay_modrate): 0;
+        my $fail  = ($fail_percent  > 0) ? rand(100) < $fail_percent  :
+                    ($fail_percent  < 0) ? !($seqn % $fail_modrate) : 0;
+        #no warnings 'uninitialized';
+        #warn "_mk_rand_callback($fail_percent:$fail_modrate, $delay_percent:$delay_modrate): seqn=$seqn fail=$fail delay=$delay";
+        if ($delay) {
+            my $msg = "DBI_GOFER_RANDOM delaying execution of $method() by $delay_duration seconds\n";
             # Note what's happening in a trace message. If the delay percent is an odd
             # number then use warn() so it's sent back to the client
             ($delay_percent % 2 == 0) ? $h->trace_msg($msg) : warn($msg);
             select undef, undef, undef, $delay_duration; # allows floating point value
         }
-        if ($fail_percent && rand(100) < $fail_percent) {
+        if ($fail) {
             undef $_; # tell DBI to not call the method
-            return $h->set_err(1, "fake error induced by DBI_GOFER_RANDOM env var");
+            return $h->set_err(1, "fake error from $method method induced by DBI_GOFER_RANDOM env var ($fail_percent%)");
         }
         return;
     }
@@ -660,6 +685,7 @@ Examples include: L<DBI::Gofer::Transport::stream> and L<DBI::Gofer::Transport::
 =head2 check_request_sub
 
 If defined, it must be a reference to a subroutine that will 'check' the request.
+It is passed the request object and the executor as its only arguments.
 
 The subroutine can either return the original request object or die with a
 suitable error message (which will be turned into a Gofer response).
@@ -667,6 +693,15 @@ suitable error message (which will be turned into a Gofer response).
 It can also construct and return a new request that should be executed instead
 of the original request.
 
+=head2 check_response_sub
+
+If defined, it must be a reference to a subroutine that will 'check' the response.
+It is passed the response object, the executor, and the request object.
+The return value is ignored, though the sub may alter the response object.
+
+This mechanism can be used to, for example, terminate the service if specific
+database errors are seen.
+
 =head2 forced_connect_dsn
 
 If set, this DSN is always used instead of the one in the request.
@@ -701,6 +736,21 @@ cached statement handles rises above this limit. The default is 1000.
 
 If true, then only the first result set will be fetched and returned in the response.
 
+=head2 forced_response_attributes
+
+A reference to a data structure that can specify extra attributes to be returned in responses.
+
+  forced_response_attributes => {
+      DriverName => {
+          dbh => [ qw(dbh_attrib_name) ],
+          sth => [ qw(sth_attrib_name) ],
+      },
+  },
+
+This can be useful in cases where the driver has not implemented the
+private_attribute_info() method and DBI::Gofer::Execute's own fallback list of
+private attributes doesn't include the driver or attributes you need.
+
 =head2 track_recent
 
 If set, specifies the number of recent requests and responses that should be
@@ -708,6 +758,10 @@ kept by the update_stats() method for diagnostics. See L<DBI::Gofer::Transport::
 
 Note that this setting can significantly increase memory use. Use with caution.
 
+=head2 forced_gofer_random
+
+Enable forced random failures and/or delays for testing. See L</DBI_GOFER_RANDOM> below.
+
 =head1 DRIVER-SPECIFIC ISSUES
 
 Gofer needs to know about any driver-private attributes that should have their
@@ -718,6 +772,59 @@ then the module fallsback to using some hard-coded details, if available, for
 the driver being used. Currently hard-coded details are available for the
 mysql, Pg, Sybase, and SQLite drivers.
 
+=head1 TESTING
+
+DBD::Gofer, DBD::Execute and related packages are well tested by executing the
+DBI test suite with DBI_AUTOPROXY configured to route all DBI calls via DBD::Gofer.
+
+Because Gofer includes timeout and 'retry on error' mechanisms there is a need
+for some way to trigger delays and/or errors. This can be done via the
+C<forced_gofer_random> configuration item, or else the DBI_GOFER_RANDOM environment
+variable.
+
+=head2 DBI_GOFER_RANDOM
+
+The value of the C<forced_gofer_random> configuration item (or else the
+DBI_GOFER_RANDOM environment variable) is treated as a series of tokens
+separated by commas.
+
+The tokens can be one of three types:
+
+=over 4
+
+=item fail=R%
+
+Set the current failure rate to R where R is a percentage.
+The value R can be floating point, e.g., C<fail=0.05%>.
+Negative values for R have special meaning, see below.
+
+=item delayN=R%
+
+Set the current random delay rate to R where R is a percentage, and set the
+current delay duration to N seconds. The values of R and N can be floating point,
+e.g., C<delay120=0.1%>.  Negative values for R have special meaning, see below.
+
+=item methodname
+
+Applies the current current random failure rate and random delay rate and duration to the named method.
+If neither a fail nor delay have been set yet then a warning is generated.
+
+=back
+
+For example:
+
+  $executor = DBI::Gofer::Execute->new( {
+    forced_gofer_random => "fail=0.01%,do,delay60=1%,execute",
+  });
+
+will cause the do() method to fail for 0.01% of calls, and the execute() method to
+fail 0.01% of calls and be delayed by 60 seconds on 1% of calls.
+
+If the percentage value (C<R>) is negative then instead of the failures being
+triggered randomly (via the rand() function) they are triggered via a sequence
+number. In other words "C<fail=-20%>" will mean every fifth call will fail.
+Each method has a distinct sequence number.
+
 =head1 AUTHOR
 
 Tim Bunce, L<http://www.linkedin.com/in/timbunce>
